Package: MCseqReplic
Version: 1.0.0
Date: 2026-04-29
Title: Monte Carlo Simulations of Time Changes in Sequences
Authors@R: c(
  person("Gilbert", "Ritschard",
    role = c("aut", "cre", "cph"),
    email = "gilbert.ritschard@unige.ch",
    comment = c(ORCID = "0000-0001-7776-0903")))
Depends: R (>= 3.0.0), TraMineR (>= 2.2-2)
Imports: WeightedCluster (>= 1.6.0), aricode, doParallel, foreach,
        doSNOW, iterators, stats, vegan, wCorr
Description: Generates replicated sets of sequences with Monte Carlo simulated timing changes and computes various indicators for evaluating effects of timing uncertainty on sequence analysis results. See Ritschard, G. and Liao, T.F. (2026): "Assessing the Impact of Timing Errors in Sequence Analysis". International Journal of Social Research Methodology <doi:10.1080/13645579.2026.2666297>.
License: GPL (>= 2)
URL: http://traminer.unige.ch
Encoding: UTF-8
Maintainer: Gilbert Ritschard <gilbert.ritschard@unige.ch>
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-04-29 07:20:07 UTC; grits
Author: Gilbert Ritschard [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-7776-0903>)
Repository: CRAN
Date/Publication: 2026-04-29 19:00:13 UTC
Built: R 4.5.2; ; 2026-05-01 18:19:16 UTC; unix
