# Generated by roxygen2: do not edit by hand

S3method(print,MCratios)
S3method(print,distMC)
S3method(summary,MCratios)
S3method(summary,distMC)
export(MCExtractDist)
export(MCclustcomp)
export(MCclustqual)
export(MCdisscorr)
export(MCdisslist)
export(MCmdscorr)
export(MCnunique)
export(MCpj)
export(MCratios)
export(MCseqReplicate)
export(MCseqdistSE)
export(MCudist)
export(seqdistMCSE)
import(TraMineR)
importFrom(TraMineR,seqdist)
importFrom(WeightedCluster,as.clustrange)
importFrom(WeightedCluster,wcAggregateCases)
importFrom(WeightedCluster,wcKMedRange)
importFrom(aricode,clustComp)
importFrom(doParallel,registerDoParallel)
importFrom(doSNOW,registerDoSNOW)
importFrom(foreach,"%:%")
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(iterators,icount)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,makePSOCKcluster)
importFrom(parallel,stopCluster)
importFrom(stats,as.dist)
importFrom(stats,dpois)
importFrom(stats,hclust)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(utils,packageVersion)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(vegan,wcmdscale)
importFrom(wCorr,weightedCorr)
